package x.y.z;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NoResultException;
import javax.persistence.Persistence;
import javax.persistence.Query;
import javax.persistence.spi.PersistenceUnitTransactionType;

import org.eclipse.persistence.config.PersistenceUnitProperties;

import com.objectdb.Enhancer;


public class MigrateODBJPA {
	
	private static final int DBMS_COMMIT_INTERVAL = 5000;
	public static void main(String[] args) throws Exception{
		migrate();
	}

	/**
	 * @return
	 */
	private static EntityManagerFactory getObjectDBEntityManagerFactory() {
		enhanceClasses();
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("objectdb://localhost:6136/test.odb;user=admin;password=admin");
		return emf;
	}
	
	private static void migrate(){
		List<Class<?>> asList = Arrays.asList(classesToMigrate());
		EntityManager source = getObjectDBEntityManagerFactory().createEntityManager();
		EntityManager dest = getEclipseLinkEntityManagerFactory().createEntityManager();
		
		for (Class<?> classToMigrate : asList) {
			Query query = source.createQuery("select max(id) from "+classToMigrate.getSimpleName());
			Long maxId = (Long) query.getSingleResult();
			
			Query entityQuery = source.createQuery("select a from "+classToMigrate.getSimpleName() +" a where a.id=:id");
			long j=0;
			boolean beginTransaction = true;
			for(long i=0; i<=maxId; i++){
				entityQuery.setParameter("id", i);
				 
				if(beginTransaction){
					dest.getTransaction().begin();
					beginTransaction = false;
				}
				
				Object entity = null;
				try{	
					entity = entityQuery.getSingleResult();
					if(entity != null){
						source.detach(entity);
						source.clear();
						dest.persist(entity);
						System.out.println("Persisted class with id: " +i + " for class " + classToMigrate.getName());
					}				
				}catch(NoResultException nre){
					System.out.println("No class with id: " +i + " for class " + classToMigrate.getName());
				}
				
				if(j==DBMS_COMMIT_INTERVAL || i==maxId){
					dest.getTransaction().commit();
					System.out.println("***************** COMMIT *****************");
					j = 0;
					beginTransaction = true;
				}
				j++;
			}
		}
	}
	
	private static Class<?>[] classesToMigrate(){
		// Put the classes you want to migrate in here!
		return new Class<?>[]{Company.class, User.class, UserPassword.class};
	}
	
	private static void enhanceClasses(){
		List<Class<?>> asList = Arrays.asList(classesToMigrate());
		StringBuilder builder = new StringBuilder();
		Iterator<Class<?>> classIterator = asList.iterator();
		while(classIterator.hasNext()){
			Class<?> theClass = classIterator.next();
			builder.append(theClass.getName());
			if(classIterator.hasNext()){
				builder.append(",");
			}
		}
		Enhancer.enhance(builder.toString());
	}
	/**
	 * @return
	 */
	private static EntityManagerFactory getEclipseLinkEntityManagerFactory() {
		Map<String, String> properties = new HashMap<String, String>();

		// Ensure RESOURCE_LOCAL transactions is used.
		properties.put(PersistenceUnitProperties.TRANSACTION_TYPE, PersistenceUnitTransactionType.RESOURCE_LOCAL.name());

		// Configure the internal EclipseLink connection pool
		properties.put(PersistenceUnitProperties.JDBC_DRIVER, "org.postgresql.Driver");
		properties.put(PersistenceUnitProperties.JDBC_URL, "jdbc:postgresql://localhost:5432/test");
		properties.put(PersistenceUnitProperties.JDBC_USER, "test");
		properties.put(PersistenceUnitProperties.JDBC_PASSWORD, "test");
		
		EntityManagerFactory emf = Persistence.createEntityManagerFactory("testPU",properties);
		return emf;
	}
}